@@ -16,4 +16,8 @@ pub enum Error {
     /// parser are out of sync.
     #[error("Internal parser error: {0}")]
     InternalParserError(String),
+
+    /// Unknown modifier.
+    #[error("Unknown modifier: {0}")]
+    UnknownModifierError(String),
 }
