@@ -15,9 +15,14 @@
 //!
 //! let grammar = Grammar::from_str(
 //!     r#"
+//!       RULES:
 //!       top = <identifier> <action>
 //!       identifier = [The #occupation#|#name#]
 //!       action = [ran|ate|sat|slept|examined the #patient#]
+//!
+//!       WEIGHTS:
+//!       name = 2
+//!       occupation = 1
 //!     "#,
 //! )
 //! .unwrap();
@@ -58,6 +63,7 @@ mod modifiers;
 mod parser;
 pub use crate::error::Error;
 use itertools::Itertools;
+use ordered_float::OrderedFloat;
 
 /// A convenience type for a `Result` of `T` or [`Error`]
 ///
@@ -124,7 +130,7 @@ impl Node {
             Node::VarRef(var) => match data.get(&var.var) {
                 Some(value) => {
                     let text = match &var.modifier {
-                        Some(modifier) => match grammar.get_modifier(&modifier) {
+                        Some(modifier) => match grammar.get_modifier(modifier) {
                             Some(modifier) => Ok(modifier(value)),
                             None => Err(Error::UnknownModifierError(modifier.to_string())),
                         },
@@ -132,7 +138,7 @@ impl Node {
                     }?;
                     Ok(vec![Expansion {
                         varrefs: vec![var.var.clone()],
-                        text: text.clone(),
+                        text,
                     }])
                 }
                 None => Ok(vec![]),
@@ -202,6 +208,7 @@ impl ToString for Node {
 pub struct Grammar {
     rules: HashMap<String, Node>,
     modifiers: HashMap<String, Rc<dyn Fn(&str) -> String>>,
+    default_weights: HashMap<String, f64>,
 }
 
 impl Grammar {
@@ -209,6 +216,7 @@ impl Grammar {
         Grammar {
             rules: HashMap::new(),
             modifiers: HashMap::new(),
+            default_weights: HashMap::new(),
         }
     }
 
@@ -230,12 +238,29 @@ impl Grammar {
             .map(|expansions| expansions.into_iter().map(|e| e.text).collect())
     }
 
-    pub fn generate(&self, name: &str, data: &HashMap<String, String>) -> Result<Option<String>> {
+    pub fn generate_with_weights(
+        &self,
+        name: &str,
+        data: &HashMap<String, String>,
+        weights: &HashMap<String, f64>,
+    ) -> Result<Option<String>> {
         let node = self.get_rule(name).unwrap();
         let mut expansions = node.expand(self, data)?;
-        expansions.sort_by_key(|e| e.varrefs.len());
+        expansions.sort_by_cached_key(|e| OrderedFloat(score_by_varref_weights(e, &weights)));
         Ok(expansions.last().map(|e| e.text.clone()))
     }
+
+    pub fn generate(&self, name: &str, data: &HashMap<String, String>) -> Result<Option<String>> {
+        self.generate_with_weights(name, data, &self.default_weights)
+    }
+}
+
+fn score_by_varref_weights(expansion: &Expansion, weights: &HashMap<String, f64>) -> f64 {
+    expansion
+        .varrefs
+        .iter()
+        .map(|varref| weights.get(varref).unwrap_or(&1.0))
+        .sum()
 }
 
 impl Default for Grammar {
