@@ -1,3 +1,5 @@
+use std::collections::HashMap;
+
 use pest::Parser;
 use pest_derive::Parser;
 
@@ -54,35 +56,61 @@ fn parse_implicit_sequence(pair: pest::iterators::Pair<Rule>) -> Result<Node> {
 
 pub(crate) fn parse_grammar(s: &str) -> Result<Grammar> {
     let mut grammar = Grammar::new();
+    let mut weights = HashMap::new();
     match GenxParser::parse(Rule::grammar, s) {
         Ok(pairs) => {
             for pair in pairs {
-                if pair.as_rule() == Rule::EOI {
-                    break;
+                match pair.as_rule() {
+                    Rule::EOI => break,
+                    Rule::rules => {
+                        for pair in pair.into_inner() {
+                            let mut inner = pair.into_inner();
+                            let lhs = inner.next().unwrap().as_str();
+                            let rhs_nodes: Result<Vec<Node>> =
+                                inner.next().unwrap().into_inner().map(parse_node).collect();
+                            let rhs = Node::Sequence(rhs_nodes?);
+                            grammar.add_rule(lhs, rhs);
+                        }
+                    }
+                    Rule::weights => {
+                        for pair in pair.into_inner() {
+                            let mut inner = pair.into_inner();
+                            let lhs = inner.next().unwrap().as_str();
+                            let weight = inner.next().unwrap().as_str().parse::<f64>().unwrap();
+                            weights.insert(lhs.to_string(), weight);
+                        }
+                    }
+                    _ => {
+                        return Err(Error::InternalParserError(format!(
+                            "Unexpected rule while parsing grammar: {:?}",
+                            pair.as_rule()
+                        )))
+                    }
                 }
-                let mut inner = pair.into_inner();
-                let lhs = inner.next().unwrap().as_str();
-                let rhs_nodes: Result<Vec<Node>> =
-                    inner.next().unwrap().into_inner().map(parse_node).collect();
-                let rhs = Node::Sequence(rhs_nodes?);
-                grammar.add_rule(lhs, rhs);
             }
         }
         Err(e) => return Err(Error::ParseError(e.to_string())),
     };
+    grammar.default_weights = weights;
     Ok(grammar)
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
+    use maplit::hashmap;
 
     #[test]
     fn test_parse() {
         let s = r#"
+          RULES:
           top = hi <name>?:[, my dear #gender#,] in <location>
           name = #name#
           location = [city of #city|capitalize#|#county# county]
+
+          WEIGHTS:
+          city = 2
+          county = 1
         "#;
         let grammar = parse_grammar(s).unwrap();
         assert_eq!(
@@ -116,5 +144,12 @@ mod tests {
                 ]),
             ])])
         );
+        assert_eq!(
+            grammar.default_weights,
+            hashmap! {
+                "city".to_string() => 2.0,
+                "county".to_string() => 1.0,
+            }
+        );
     }
 }
